/**
 * Remote collection
 * @param workspace - Remote workspace
 * @param nameOrKey - Name or key of the collection
 */
declare class Collection {
    constructor(workspace: Workspace, nameOrKey: string);
    /**
     * Populate info and schemas for this collection from server.
     */
    open(): Promise<void>;
    /**
     * Close the collection.
     *
     * Properly closes open HTTP requests.
     */
    close(): void;
    /**
     * Put a new feed to the collection.
     * @param key - The hex-encoded key of the feed to add.
     * @param [info] - Optional information about the feed.
     *                          TODO: Document
     */
    putFeed(key: string, info?: any): void;
    addFeed(): void;
    /**
     * Query the collection.
     *
     * Returns an array of matching records.
     *
     * Records may have a meta property that includes query-specific metadata (e.g. the score for search queries).
     * @param name - The name of a supported query.
     *                        Supported queries that ship with @arsonar/core are:
     *                        records, search, relations, history and indexes.
     * @param args - The arguments for the query. Depends on the query being used.
     *                        For records: `{ schema, name, id }`
     *                        For history: `{ from: timestamp, to: timestamp }`
     *                        For search: Either a "string" for a simple full-text search, or a
     *                           tantivy query object.
     *                        For indexes: `{ schema, prop, value, from, to, reverse, limit }`
     *                           (to be documented)
     *                        For relations: `{ subject, object, predicate }`
     *                           where subject and object are ids and predicate is `type#field`
     * @param [opts] - Optional options
     * @param [opts.sync = false] - Wait for all pending indexing operations to be finished.
     * @returns A promise that resolves to an array of record objects.
     */
    query(name: string, args: any, opts?: {
        sync?: boolean;
    }): Promise<Record[]>;
    /**
     * Put a new record into the collection.
     * @param record - The record.
     * @param record.schema - The schema of the record.
     * @param [record.id] - The entity id of the record. If empoty an id will be created.
     * @param record.value - Value of the record.
     * @returns An object with an `{ id }` property.
     */
    put(record: {
        schema: string;
        id?: string;
        value: any;
    }): Promise<object>;
    /**
     * Get records by their semantic address (type and id) or by their storage address (key and seq).
     * @param req - The get request. Either `{ type, id }` or `{ key, seq }`.
     * @param [opts] - Optional options.
     * @param [opts.sync = false] - Wait for all pending indexing operations to be finished.
     * @returns A promise that resolves to an array of record objects.
     */
    get(req: any, opts?: {
        sync?: boolean;
    }): Promise<object[]>;
    /**
     * Get a specific version of a record.
     * @param address - The block address of the record version `feedkey@seq`
     *    where `feedkey` is the hex-encoded public key of a feed and `seq` is a sequence number (uint).
     */
    getVersion(address: string): void;
    /**
     * Deletes a record.
     * @param record - The record to delete. Has to have `{ id, type }` properties set.
     * @returns - An object with `{ id, type }` properties of the deleted record.
     */
    del(record: any): Promise<object>;
    /**
     * Add a new type to the collection.
     * @param schema - A schema object.
     * @returns A promise that resolves to the saved schema object.
     */
    putType(schema: any): Promise<object>;
    /**
     * Create a writable stream to put records into the collection.
     *
     * Example:
     * ```javascript
     * const batchStream = collection.createBatchStream()
     * batch.write(record)
     * batch.close()
     * ```
     * @returns A writable stream
     */
    createBatchStream(): Writable<Record>;
    /**
     * Wait for all pending indexing operations to be finished.
     */
    sync(): Promise<void>;
    /**
     * Subscribe to events on this collection.
     *
     * Returns a readable stream that emits event objects.
     * They look like this:
     * `{ event: string, data: object }`
     *
     * Events are:
     *
     * * `update`: with data `{ lseq }`
     * * `feed`: with data `{ key }`
     * * `schema-update`
     */
    createEventStream(): Readable<object>;
    /**
     * Pull live updates from the server as they happen.
     *
     * After calling this method once, all new records and record versions
     * are pulled from the server once available. The `update` event
     * is emitted when new records are about to arrive.
     */
    pullLiveUpdates(): void;
    /**
     * Subscribe to this collection.
     *
     * This will fetch all records from the first to the last and then waits for new records.
     * Currently only intended for usage in bots (not in short-running Browser clients).
     * @param Subscription - name. Has to be unique per running Sonar instance
     * @param Async - callback function that will be called for each incoming record
     */
    subscribe(Subscription: string, Async: (...params: any[]) => any): void;
    /**
     * Reindex the secondary indexes (views) in this collection.
     *
     * Use with care, this can be expensive.
     * @param Optional - array of view names to reindex.
     *  If unset all views will be reindexed.
     */
    reindex(Optional: string[]): void;
}

/**
 * File system for a collection.
 * @param collection - Collection
 */
declare class Fs {
    constructor(collection: Collection);
    /**
     * Resolve a file URL into a HTTP url.
     * @param path - Either a hyper:// URL or a path in the collection's file system.
     * @returns The HTTP URL to the file.
     */
    resolveURL(path: string): string;
    /**
     * List the drives that are part of this collection.
     * @returns Array of drive objects with keys `{ alias, key, writable }`
     */
    listDrives(): Promise<object[]>;
    /**
     * Get the contents of a directory.
     * @param path - A hyper:// URL or a relative path within the collection's file system.
     * @param [opts] - Options
     * @param [opts.includeStats = true] - Include metadata for each file
     * @returns An array of objects with file metadata.
     */
    readdir(path: string, opts?: {
        includeStats?: string;
    }): Promise<object[]>;
    /**
     * Write a file
     * @param path - A hyper:// URL or a relative path within the collection's file system.
     * @param file - File content to write. Can be either a buffer or a read stream.
     * @param [opts] - Options.
     */
    writeFile(path: string, file: Stream | Buffer, opts?: any): Promise;
    /**
     * Read a file into a buffer.
     * @param path - A hyper:// URL or a relative path within the collection's file system.
     * @param [opts] - Options. TODO: document.
     * @returns The file content. A Buffer object in Node.js, a ArrayBuffer object in the browser.
     */
    readFile(path: string, opts?: any): Promise<ArrayBuffer | Buffer>;
    /**
     * Get a read stream for a file.
     * @param path - A hyper:// URL or a relative path within the collection's file system.
     * @param [opts] - Options. TODO: document.
     * @returns A `stream.Readable` in Node.js, a `ReadableStream`in the browser.
     */
    createReadStream(path: string, opts?: any): Promise<ReadableStream | Readable>;
    /**
     * Get metadata about a file.
     * @param path - A hyper:// URL or a relative path within the collection's file system.
     * @returns A plain object with the stat info.
     */
    statFile(path: string): Promise<object>;
}

/**
 * The manager for a remote connection to a Sonar workspace.
 * @param [opts] - Optional options.
 * @param [opts.url = http://localhost:9191/api/v1/default] - The API endpoint to talk to.
 * @param [opts.accessCode] - An access code to login at the endpoint.
 * @param [opts.token] - A JSON web token to authorize to the endpoint.
 * @param [opts.name] - The name of this client.
 */
declare class Workspace {
    constructor(opts?: {
        url?: string;
        accessCode?: string;
        token?: string;
        name?: string;
    });
    /**
     * Closes the client.
     */
    close(): Promise<void>;
    /**
     * Get a list of all collections available on this endpoint.
     * @returns Promise that resolves to an array of collection info objects.
     */
    listCollections(): Promise<object[]>;
    /**
     * Creates a collection with name name on the Sonar server. The name may not contain whitespaces. opts is an optional object with:
     * @param name - Name of the new collection, may not contain whitespaces.
     * @param [opts] - Optional options object.
     * @param [opts.key] - Hex string of an existing collection. Will then sync this collection instead of creating a new, empty collection.
     * @param [opts.alias] - When setting key, alias is required and is your nick name within this collection.
     * @returns The created collection.
     */
    createCollection(name: string, opts?: {
        key?: string;
        alias?: string;
    }): Promise<Collection>;
    /**
     * Updates the config of a collection.
     * @param name - Name of the collection.
     * @param info - [TODO:description]
     * @param info.share - Controls whether a collection is shared via p2p.
     */
    updateCollection(name: string, info: {
        share: boolean;
    }): Promise<void>;
    /**
     * Returns a Collection object for a given key or name of a collection.
     * @param keyOrName - Key or name of the collection to open/return.
     */
    openCollection(keyOrName: string): Promise<Collection>;
    /**
     * Fetch a resource.
     *
     * This is a wrapper around the fetch web API. It should be API compatible to fetch,
     * with the following changes:
     * @param [opts.requestType = 'json'] - Request encoding and content type.
     *   Supported values are 'json' and 'binary'
     * @param [opts.responseType = 'text'] - Response encoding. If the response
     *    has a JSON content type, will always be set to 'json'.
     *    Supported values are 'text', 'binary' and 'stream'.
     * @param [opts.params] - Query string parameters (will be encoded correctly).
     * @returns If the response has a JSON content type header, the
     *    decoded JSON will be returned. if opts.responseType is 'binary' or 'text',
     *    the response will be returned as a buffer or text.
     *
     * TODO: Rethink the default responseType cascade.
     */
    fetch(): Promise<object>;
}

